// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "component/world/World.hpp"
#include "component/entity/EntityID.hpp"
#include "component/entity/EntityTilePosition.hpp"

namespace Dawn {
  class Map : public SceneComponent {
    private:
      std::unordered_map<enum EntityID, std::weak_ptr<Entity>> entities;

      /**
       * Part of the ECS, this function is called when an entity is initialized.
       * 
       * @param item Scene Item that has been initialized.
       */
      void entityNotifyInit(std::shared_ptr<SceneItem> item);

      /**
       * Part of the ECS, this function is called when an entity is disposed.
       * 
       * @param item Scene Item that has been disposed.
       */
      void entityNotifyDispose(std::shared_ptr<SceneItem> entity);

    public:
      std::shared_ptr<World> world;

      void onInit() override;
      void onDispose() override;

      /**
       * Gets the entity with the given ID.
       * 
       * @param id The ID of the entity to get.
       * @return The entity with the given ID.
       */
      std::shared_ptr<Entity> getEntity(const EntityID id);

      /**
       * Gets the entity at the given position.
       * 
       * @param pos The position to check for an entity.
       * @return The entity at the given position, or nullptr.
       */
      std::shared_ptr<Entity> getEntityAt(const EntityTilePosition &pos);

      friend class Entity;
  };
}